(** Executable: interprets stdin as mail message and posts content to 
  specified HTTP address *)

open StdLabels
open MoreLabels
open Printf
module Unix = UnixLabels
module Map = PMap.Map
module Set = PSet.Set

let logfile = open_out "/home/sks/addlog"

(** Argument parsing *)

let anonymous = ref []

let usage_string = 
  Sys.argv.(0) ^ " sks_directory_name"

let anon_options option = 
  anonymous := option::!anonymous

let parse_spec = [ ]

let dirname = 
  Arg.parse parse_spec anon_options usage_string;
  if List.length !anonymous <> 1 
  then (
    printf "Wrong number (%d) of arguments given.  %s\n" 
	  (List.length !anonymous)
	  usage_string;
    exit (-1)
  ) else
    Filename.concat (List.hd !anonymous) "messages"

(*********************************************)


(** Simple I/O routines *)
let max_filesize = 200 * 1024
let buffer = String.create max_filesize

let input_msg inchan = 
  let pos = ref 0 in
  let eof = ref false in
  while !pos < String.length buffer && not !eof do
    let read = input inchan buffer !pos (String.length buffer - !pos) in
    pos := !pos + read;
    if read = 0 then eof := true
  done;
  String.sub buffer ~pos:0 ~len:!pos


let input_string inchan length = 
  let buf = Buffer.create length in
  Buffer.add_channel buf inchan length;
  Buffer.contents buf

let protect ~f ~finally = 
  let result = ref None in
  try 
    result := Some (f ());
    raise Exit
  with
      Exit as e -> 
	finally ();
	(match !result with Some x -> x | None -> raise e)
    | e -> 
	finally ();
	raise e

let run () =
  let msgtext = input_msg stdin in
  if not (Sys.file_exists dirname)
  then Unix.mkdir dirname 0o700;
  let fname = sprintf "msg-%08d" (Random.int 100000000) in
  let fname = Filename.concat dirname fname in
  let f = open_out fname in
  output_string f msgtext;
  close_out f;
  Sys.rename fname (fname ^ ".ready")

let () = 
  Random.self_init ();
  run ()
