(** Loads settings from settings file. *)

open StdLabels
open MoreLabels
open Printf
open Settings

let protect ~f ~finally = 
  let result = ref None in
  try 
    result := Some (f ());
    raise Exit
  with
      Exit as e -> 
	finally (); 
	(match !result with Some x -> x | None -> raise e)
    | e -> 
	finally (); raise e


let whitespace c = c = '\t' || c = ' ' || c = '\n' 

let strip s = 
  let lower = ref 0 in
  while !lower < String.length s && whitespace s.[!lower] do
    incr lower
  done;

  let upper = ref (String.length s - 1) in
  while !upper >= 0 && whitespace s.[!upper] do
    decr upper
  done;
  
  if !upper < !lower then ""
  else
    String.sub s ~pos:!lower ~len:(!upper - !lower + 1)

let csplit c s = 
  let i = String.index s c in
  (strip (String.sub ~pos:0 ~len:i s),
   strip (String.sub ~pos:(i+1) ~len:(String.length s - i - 1) s)
  )

let rec parse_settings_file ?(line=1) actions file = 
  match (try Some (input_line file) 
	 with End_of_file -> None)
  with
      None -> ()
    | Some l ->
	begin
	  try (
	    if String.length l = 0 || l.[0] = '#' 
	    then raise Not_found;
	    let (key,value) = csplit ':' l in
	    match List.assoc key actions with
	      | Arg.Unit f -> f ()
	      | Arg.Set flag -> flag := true
	      | Arg.Clear flag -> flag := false
	      | Arg.String f -> f value
	      | Arg.Int f -> f (int_of_string value)
	      | Arg.Float f -> f (float_of_string value)
	      | Arg.Rest f -> f value
	  ) with
	      Not_found -> ()
	    | e -> 
		eprintf "Parse error at line %d\n" line;
		raise e
		
	end;
	parse_settings_file ~line:(line + 1) actions file



let actions = 
  List.map ~f:(fun (key,arg,dstr) ->
		 (String.sub ~pos:1 ~len:(String.length key - 1) key,
		  arg)
	      )
    parse_spec


let default_fname = "sksconf"
let file_loaded = ref false

let load_settings fname = 
  let file = open_in fname in
  protect ~f:(fun () -> parse_settings_file actions file)
    ~finally:(fun () -> close_in file)

let file_exists fname = 
  try
    ignore (Unix.stat fname);
    true
  with
      Unix.Unix_error (Unix.ENOENT,_,_) -> false

let extra_pspec = ("-scanline", Arg.String load_settings, "Load settings from file")

let _ = 
  if file_exists default_fname then (
    load_settings default_fname;
  );
  Arg.parse (extra_pspec::parse_spec)
      anon_options usage_string;
  anonlist := List.rev !anonlist;
  prepare_options ()

  
