open Printf
open StdLabels
open MoreLabels
module Unix = UnixLabels
open Unix

open Packet

let page ~title ~body = 
  sprintf 
    "<html><header><title>%s</title></header></html>\r\n<body><h1>%s</h1>%s</body>" 
    title title body

let link ~hostname ~port ~keyid =
  sprintf "http://%s:%d/pks/lookup?op=get&search=0x%s"
    hostname port keyid

let keyinfo_header = "    bits Fprint   Cr. Date   UserID"

let keyinfo pki ~keyid ~link ~userids = 
  let tm = localtime (Int32.to_float pki.pk_ctime) in
  let base = 
    sprintf "%2d %4d <a href=\"%s\">%8s</a> %4d/%02d%02d "
      pki.pk_version pki.pk_keylen link keyid 
      (1900 + tm.tm_year) tm.tm_mon tm.tm_mday 
  in
  let uidstr = String.concat ~sep:"\r\n                           " userids in
  base ^ uidstr

let keyinfo_pks pki ~keyid ~link ~userids = 
  let tm = localtime (Int32.to_float pki.pk_ctime) in
  let base = 
    sprintf "pub %4d/<a href=\"%s\">%8s</a> %4d/%02d/%02d "
      pki.pk_keylen link keyid 
      (1900 + tm.tm_year) tm.tm_mon tm.tm_mday 
  in
  let uidstr = String.concat ~sep:"\r\n                             " userids in
  base ^ uidstr



let fingerprint ~fp = 
  sprintf "\t Fingerprint=%s" fp

let preformat_list elements = 
  sprintf "<pre>%s</pre>"
    (String.concat ~sep:"\r\n" elements ^ "\r\n")
