open StdLabels
open MoreLabels
module Unix=UnixLabels

open Printf

module type MsgMarshal =
sig
  type msg_t
  val marshal: Channel.out_channel_obj -> msg_t -> unit
  val unmarshal: Channel.in_channel_obj -> msg_t
  val to_string: msg_t -> string
  val print: string -> unit
  val debug : bool ref
end

module Container =
  functor (Msg:MsgMarshal) ->
struct

  type msg_container = 
      { msg: Msg.msg_t;
	(* nonce: int; *)
      }

  let marshal_noflush cout msg = 
    if !Msg.debug then
      Msg.print (sprintf "Marshalling: %s" (Msg.to_string msg));
    Msg.marshal cout#upcast msg

  let marshal cout msg = 
    marshal_noflush cout msg;
    cout#flush

  let unmarshal cin = 
    let msg = Msg.unmarshal cin#upcast in
    if !Msg.debug then
      Msg.print (sprintf "Unmarshalling: %s" (Msg.to_string msg));
    { msg = msg; }

end


