module type DBSettings =
sig
  val withtxn : bool
  val cache_bytes : int option
  val pagesize : int option
  val dbdir : string
  val dumpdir : string
end

module Make :
  functor (Dbs : DBSettings) ->
sig
  type 'a offset = { fnum : int; pos : 'a; } 
  and skey =
      KeyString of string
    | Key of Packet.packet list
    | Offset of int offset
    | LargeOffset of int64 offset
  and dbdump = { directory : string; filearray : in_channel array; } 
  and dbstate = {
    dbenv : Bdb.Dbenv.t;
    key : Bdb.Db.t;
    word : Bdb.Db.t;
    keyid : Bdb.Db.t;
    time : Bdb.Db.t;
    tqueue : Bdb.Db.t;
    dump : dbdump;
  } 
  val dbstate : dbstate option ref
  exception No_db
  val get_dbs : unit -> dbstate
  val get_dump_filearray : unit -> in_channel array
  val marshal_offset : < write_int : int -> 'a; .. > -> int offset -> 'a
  val unmarshal_offset : < read_int : int; .. > -> int offset
  val marshal_large_offset :
    < write_int : int -> 'a; write_int64 : 'b -> 'c; .. > ->
      'b offset -> 'c
  val unmarshal_large_offset :
    < read_int : int; read_int64 : 'a; .. > -> 'a offset
  val skey_of_string : string -> skey
  val skey_to_string : skey -> string
  val keystring_of_skey : skey -> string
  val keystring_of_string : string -> string
  val key_of_skey : skey -> Packet.packet list
  val key_to_string : Packet.packet list -> string
  val key_of_string : string -> Packet.packet list
  val read_dir_suff : string -> string -> string list
  val open_dbs : unit -> unit
  val open_dump : string -> unit
  val close_dbs : unit -> unit
  val sync : unit -> unit
  val checkpoint : unit -> unit
  val is_alnum : char -> bool
  val extract_words_rec :
    string -> start:int -> len:int -> string Set.t -> string Set.t
  val extract_word_set : string -> string Set.t
  val extract_words : string -> string list
  val key_extract_words : Packet.packet list -> string list
  val float_to_string : float -> string
  val float_of_string : string -> float
  val event_to_string : Common.event -> string
  val event_of_string : string -> Common.event
  val flatten_array_of_lists : 'a list array -> 'a array
  val flatten_array_of_lists :
    (string * string) list array -> (string * string) array
  val jcursor_get_all : ?max:int -> Bdb.Cursor.t -> string list
  val get_by_words : max:int -> string list -> Packet.packet list list
  val get_by_time : max:'a -> float -> string list
  val has_hash : string -> bool
  val get_skeystring_by_hash : string -> string
  val get_keystring_by_hash : string -> string
  val get_by_hash : string -> Packet.packet list
  val check_word_hash_pair : word:string -> hash:string -> bool
  val check_keyid_hash_pair : keyid:string -> hash:string -> bool
  val delete_key :
    ?txn:Bdb.txn -> ?hash:Digest.t -> Packet.packet list -> unit
  val get_keystrings_by_hashes : string list -> string list
  val iter : f:(key:string -> data:string -> 'a) -> unit
  val keyiter : f:(string -> 'a) -> unit
  val get_hashes_by_keyid : string -> string list
  val get_skeystrings_by_keyid : string -> string list
  val get_by_keyid : string -> Packet.packet list list
  val logquery : ?maxsize:int -> float -> (float * Common.event) list
  val create_hashstream : unit -> string SStream.sstream * (unit -> unit)
  val create_hash_skey_stream :
    unit -> (string * string) SStream.sstream * (unit -> unit)
  val last_ts : unit -> float
  val add_key_internal :
    ?txn:Bdb.txn -> ?hash:Digest.t -> Packet.packet list -> unit
  val add_key :
    ?parent:Bdb.Txn.t -> ?hash:Digest.t -> Packet.packet list -> unit
  val add_multi_keys : Packet.packet list list -> unit
  type key_metadata = {
    md_hash : string;
    md_words : string list;
    md_keyid : string;
    md_time : float;
    md_skey : skey;
  } 
  val shorten_offset : int64 offset -> skey
  val key_to_metadata_large_offset :
    int64 offset -> Packet.packet list -> key_metadata
  val key_to_metadata_offset :
    int offset -> Packet.packet list -> key_metadata
  val key_to_metadata : Packet.packet list -> key_metadata
  val add_keys_internal : ?txn:Bdb.txn -> Packet.packet list list -> unit
  val add_keys : Packet.packet list list -> unit
  val add_mds : key_metadata list -> unit
  val add_key_merge_internal :
    ?txn:Bdb.txn -> ?hash:Digest.t -> Packet.packet list -> unit
  val add_key_merge :
    ?parent:Bdb.Txn.t -> ?hash:Digest.t -> Packet.packet list -> unit
  val add_multi_keys_merge : Packet.packet list list -> unit
end
